return {
  "neovim/nvim-lspconfig",
  event = { "BufReadPre", "BufNewFile" },
  dependencies = {
    "hrsh7th/cmp-nvim-lsp",
    { "antosha417/nvim-lsp-file-operations" },
  },
  config = function()
    -- import lspconfig plugin
    local lspconfig = require("lspconfig")

    -- import cmp-nvim-lsp plugin
    local cmp_nvim_lsp = require("cmp_nvim_lsp")

    local map = vim.keymap -- for conciseness

    local opts = { noremap = true, silent = true }
    local on_attach = function(client, bufnr)
      opts.buffer = bufnr

      -- set keymaps
      opts.desc = "Show LSP references"
      map.set("n", "gR", "<cmd>Telescope lsp_references<CR>", opts, { desc = "show definition, references" })

      opts.desc = "Go to declaration"
      map.set("n", "gD", "vim.lsp.buf.declaration", opts, { desc = "go to declaration" })

      opts.desc = "Show LSP definitions"
      map.set("n", "gd", "<cmd>Telescope lsp_definitions<CR>", opts, { desc = "show lsp definitions" })

      opts.desc = "Show LSP implementations"
      map.set("n", "gi", "<cmd>Telescope lsp_implementations<CR>", opts, { desc = "show lsp implementations" })

      opts.desc = "Show LSP type definitions"
      map.set("n", "gt", "<cmd>Telescope lsp_type_definitions<CR>", opts, { desc = "show lsp type implementations" })

      opts.desc = "See available code actions"
      map.set({ "n", "v" }, "<leader>ca", vim.lsp.buf.code_action, opts, { desc = "see available code actions" })

      opts.desc = "Smart rename"
      map.set("n", "<leader>rn", vim.lsp.buf.rename, opts, { desc = "smart rename" })

      opts.desc = "Show buffer diagnostics"
      map.set("n", "<leader>D", "<cmd>Telescope diagnostics bufnr=0<CR>", opts, { desc = "show diagnostics for file" })

      opts.desc = "Show line diagnostics"
      map.set("n", "<leader>d", vim.diagnostic.open_float, opts, { desc = "show diagnostics for line" })

      opts.desc = "Go to previous diagnostic"
      map.set("n", "[d", vim.diagnostic.goto_prev, opts, { desc = "jump to prev diagnostic in buffer" })

      opts.desc = "Go to next diagnostic"
      map.set("n", "]d", vim.diagnostic.goto_next, opts, { desc = "jump to next diagnostic in buffer" })

      opts.desc = "Show documentation for what is under cursor"
      map.set("n", "K", vim.lsp.buf.hover, opts, { desc = "show documentation for what is under cursor" })

      opts.desc = "Restart LSP"
      map.set("n", "<leader>rs", ":LspRestart<CR>", opts, { desc = "mapping to restart lsp if necessary" })
    end

    -- used to enable autocompletion (assign to every lsp server config)
    local capabilities = cmp_nvim_lsp.default_capabilities()

    -- Change the Diagnostic symbols in the sign column (gutter)
    local signs = { Error = " ", Warn = " ", Hint = "󰠠 ", Info = " " }
    for type, icon in pairs(signs) do
      local hl = "DiagnosticSign" .. type
      vim.fn.sign_define(hl, { text = icon, texthl = hl, numhl = "" })
    end

    -- configure html server
    lspconfig["html"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
    })

    -- configure typescript server
    lspconfig["tsserver"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
    })

    -- configure css server
    lspconfig["cssls"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
    })

    -- configure emmet language server
    lspconfig["emmet_ls"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
      filetypes = { "html", "typescriptreact", "javascriptreact", "css", "sass", "scss", "less", "svelte" },
    })

    -- configure python server
    lspconfig["pyright"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
    })

    -- configure bash server
    lspconfig["bashls"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
    })

    -- configure lua server (with special settings)
    lspconfig["lua_ls"].setup({
      capabilities = capabilities,
      on_attach = on_attach,
      settings = { -- custom settings for lua
        Lua = {
          -- make the language server recognize "vim" global
          diagnostics = {
            globals = { "vim" },
          },
          workspace = {
            -- make language server aware of runtime files
            library = {
              [vim.fn.expand("$VIMRUNTIME/lua")] = true,
              [vim.fn.stdpath("config") .. "/lua"] = true,
            },
          },
        },
      },
    })
    lspconfig["svelte"].setup({
      capabilities = capabilities,
      on_attach = function(client, bufnr)
        on_attach(client, bufnr)

        vim.api.nvim_create_autocmd("BufWritePost", {
          pattern = { "*.js", "*.ts" },
          callback = function(ctx)
            if client.name == "svelte" then
              client.notify("$/onDidChangeTsOrJsFile", { uri = ctx.file })
            end
          end,
        })
      end
    })
  end,
}
